# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand


# Tests for identifyTopFit function
test_that("identifyTopFit correctly identifies top-fitting objects", {
    input_var_list <- init_variable()
    ## -- simulate RNAseq data 
    set.seed(101)
    mock_data <- mock_rnaseq(input_var_list, 
                         n_genes = 5,
                       min_replicates  = 3,
                       max_replicates = 3,
                       basal_expression = 2,
                       sequencing_depth = 1e5)
    ## -- prepare data & fit a model with mixed effect
    data2fit = prepareData2fit(countMatrix = mock_data$counts, 
                         metadata =  mock_data$metadata, 
                         normalization = T)
    l_tmb <- fitModelParallel(formula = kij ~ myVariable, data = data2fit, 
                     group_by = "geneID", family = glmmTMB::nbinom2(link = "log"), 
                     n.cores = 1)
    glance_tmb(l_tmb)
    # Identify top fitting observations based on AIC with MAD filtering
    top_genes <- identifyTopFit(l_tmb, metric = "AIC", filter_method = "mad", keep = "top", 
                   sort = TRUE, decreasing = TRUE, mad_tolerance = 3)
    expect_equal(top_genes, c("gene5", "gene4", "gene1", "gene2", "gene3"))
    # Identify low fitting observations based on BIC without sorting
    top_genes <-identifyTopFit(l_tmb, metric = "BIC", filter_method = "mad", keep = "low", sort = FALSE)
    expect_equal(top_genes, character())

    # Identify top fitting observations based on log-likelihood with MAD filtering and custom tolerance
    top_genes <- identifyTopFit(l_tmb, metric = "logLik", filter_method = "mad", keep = "top", mad_tolerance = 2)
    expect_equal(top_genes, c("gene1", "gene2", "gene3", "gene4"))
})

# Tests for get_mad_left_threshold function
test_that("get_mad_left_threshold correctly calculates MAD-based left threshold", {
  left_threshold <- get_mad_left_threshold(c(100, 200, 300), 3)
  expect_equal(left_threshold, 200 - (3 * 100))
})


# Tests for get_mad_message
test_that('get_mad_user_message return correct output', {
  expect_message(get_mad_user_message("BIC", c(50, 75, 100), 75, "low"))
  expect_message(get_mad_user_message("BIC", c(50, 75, 100), 75, "top"))
})


