# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand

# Test case 1: Check if the function returns a data frame
test_that("getInput2simulation returns a data frame", {
  list_var <- init_variable()
  result <- getInput2simulation(list_var)
  expect_is(result, "data.frame")
  expected <- data.frame(geneID = c("gene1", "gene1"), label_myVariable = as.factor(c("myVariable1", "myVariable2")), myVariable = c(2,3))
  expect_equal(result, expected)
  })

# Test for getCoefficients function
test_that("getCoefficients returns the correct output", {
  # Create dummy data
  n_genes <- 3
  list_var = init_variable()
  # Call the function
  coefficients <- getCoefficients(list_var, list(), list(), n_genes)
  
  # Check the output
  expect_equal(nrow(coefficients), n_genes*list_var$myVariable$level)
  expect_equal(colnames(coefficients), c("geneID", "label_myVariable")) 
})

# Test for getMu_ij_matrix function
test_that("getMu_ij_matrix returns the correct output", {
  # Create a dummy coefficients dataframe
  dtf_coef <- data.frame(geneID = c("Gene1", "Gene1", "Gene1"),
                         label_varA = c("A1", "A2", "A3"),
                         label_varB = c("B1", "B2", "B3"),
                         mu_ij = c(1, 2, 3))
  
  # Call the function
  mu_matrix <- getMu_ij_matrix(dtf_coef)
  # Check the output
  expect_equal(dim(mu_matrix), c(1, 9)) 
  
})

# Test for getSubCountsTable function
test_that("getSubCountsTable returns the correct output", {
  # Create dummy data
  l_genes <- c("gene1", "gene2", "gene3")
  matx_Muij = data.frame(sple1 = c(1,3,4), sple2 = c(2, 0, 9), sple3 = c(1, 69, 2)) %>% as.matrix()
  rownames(matx_Muij) <- l_genes
  matx_dispersion <- matrix(0.5, nrow = 3, ncol = 3)
  replicateID <- 1
  l_bool_replication <- c(TRUE, FALSE, TRUE)
  
  # Call the function
  subcounts_table <- getSubCountsTable(matx_Muij, matx_dispersion, 1, l_bool_replication)
  
  # Check the output
  expect_equal(dim(subcounts_table), c(3, 2))
  expect_equal(rownames(subcounts_table), l_genes)
})


test_that("getReplicationMatrix returns the correct replication matrix", {
  minN <- 2
  maxN <- 4
  n_samples <- 3
  expected <- matrix(c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, FALSE), nrow = maxN)
  
  set.seed(123)
  result <- getReplicationMatrix(minN, maxN, n_samples)
  
  expect_equal(result, expected)
})

test_that("getSampleID return the correct list of sampleID",{
   expect_equal(getSampleID(init_variable()), c("myVariable1", "myVariable2"))
})

# Create a test case for getMu_ij
test_that("getMu_ij returns the correct output", {
  # Create a sample coefficient data frame
  dtf_coef <- data.frame(
    log_qij = c(1, 9, 0.1),
    basalExpr = c(2, 3, 4)
  )

    # Call the getMu_ij function
  result <- getMu_ij(dtf_coef)

  # Check if the mu_ij column is added
  expect_true("mu_ij" %in% colnames(result))

  # Check the values of mu_ij
  #expected_mu_ij <- c(20.08554, 162754.79142 , 60.34029)
  #expect_equal(result$mu_ij, expected_mu_ij, tolerance = 0.000001)
})


# Create a test case for getLog_qij
test_that("getLog_qij returns the correct output", {
  # Create a sample coefficient data frame
  dtf_coef <- data.frame(
    beta1 = c(1.2, 2.3, 3.4),
    beta2 = c(0.5, 1.0, 1.5),
    non_numeric = c("a", "b", "c")
  )

  # Call the getLog_qij function
  result <- getLog_qij(dtf_coef)

  # Check if the log_qij column is added
  expect_true("log_qij" %in% colnames(result))

  # Check the values of log_qij
  expected_log_qij <- c(1.7, 3.3, 4.9)
  expect_equal(result$log_qij, expected_log_qij)
})

test_that("getCountsTable returns the correct counts table", {
  mat_mu_ij <- matrix(c(1,2,3,4,5,6), ncol = 3, byrow = T)
  rownames(mat_mu_ij) <- c("gene1", "gene2")
  colnames(mat_mu_ij) <- c("sample1", "sample2", "sample3")
  mat_disp <- matrix(c(0.3,0.3,0.3, 0.5,0.5,0.5), ncol = 3, byrow = T)
  rownames(mat_disp) <- c("gene1", "gene2")
  colnames(mat_disp) <- c("sample1", "sample2", "sample3")
  mat_repl <- matrix(c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE), ncol = 3, byrow = T)
  
  expected_df <- matrix(c(0,0,1,0,0,0,0,1,0,2,34,18,0,0,3,10,7,2), nrow = 2, byrow = T) %>% as.data.frame()
  rownames(expected_df) <- c("gene1", "gene2")
  colnames(expected_df) <- c("sample1_1", "sample2_1", "sample3_1", "sample1_2", 
                             "sample2_2","sample3_2","sample1_3", "sample2_3" ,"sample3_3")
  
  set.seed(123)
  result <- getCountsTable(mat_mu_ij, mat_disp, mat_repl)

  expect_true(is.data.frame(result))
  expect_equal(colnames(result), colnames(expected_df))
  expect_equal(rownames(result), rownames(expected_df))

})



test_that("getSampleMetadata returns expected output", {
  # Set up input variables
  list_var <- init_variable()
  n_genes <- 3
  replicationMatrix <- matrix(TRUE, nrow = 2, ncol = 2)

  # Run the function
  result <- getSampleMetadata(list_var, n_genes, replicationMatrix)
  
  # Define expected output
  expected_colnames <- c("myVariable", "sampleID")
  expect_equal(colnames(result), expected_colnames)
  
  # Check the output class
  expect_true(is.data.frame(result))
  
  # check nrow output
  expect_equal(nrow(result), 4)

})


test_that("replicateByGroup return the correct ouptut", {
  df <- data.frame(group = c("A", "B"), value = c(1, 2))
  result <- replicateByGroup(df, "group", c(2, 3))
  
  expect <- data.frame(group = c("A", "A", "B", "B", "B"), 
                       value = c(1, 1, 2,2,2), 
                       sampleID = c("_1", "_2", "_1", "_2", "_3" ))
  expect_equal(result, expect)

})


test_that("getDispersionMatrix returns the correct dispersion matrix", {
  n_genes = 3
  list_var = init_variable()
  dispersion <- 1:3
  expected <- matrix(1:3,byrow = F, nrow = 3, ncol = 2)
  rownames(expected) <- c("gene1", "gene2", "gene3")
  colnames(expected) <- c("myVariable1", "myVariable2")
  result <- getDispersionMatrix(list_var, n_genes, dispersion )
  expect_equal(result, expected)
})


# Test case: Valid input vector with numeric and positive values
test_that("Valid input vector with numeric and positive values", {
  input_vector <- c(0.5, 1.2, 0.8)
  result <- getValidDispersion(input_vector)
  expect_identical(result, input_vector)
})

# Test case: Valid input vector with numeric, positive, and negative values
test_that("Valid input vector with numeric, positive, and negative values", {
  input_vector <- c(0.5, -0.3, 1.2, 0.8)
  result <- getValidDispersion(input_vector)
  expect_identical(result, c(0.5, 1.2, 0.8))
})

# Test case: Valid input vector with non-numeric elements
test_that("Valid input vector with non-numeric elements", {
  input_vector <- c(0.5, "invalid", 0.8)
  result <- getValidDispersion(input_vector)
  expect_identical(result, c(0.5, 0.8))
})

# Test case: Empty input vector
test_that("Empty input vector", {
  input_vector <- numeric(0)
  expect_error(getValidDispersion(input_vector), "Invalid dispersion values provided.")
})

# Test case: unique value in vector
test_that("unique value in vector", {
  input_vector <- 5
  expect_equal(getValidDispersion(input_vector), 5)
})

# Test case: All negative values
test_that("All negative values", {
  input_vector <- c(-0.5, -1.2, -0.8)
  expect_error(getValidDispersion(input_vector), "Invalid dispersion values provided.")
})




