# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand

# Test updateParallel function
test_that("updateParallel function returns correct results", {
  # Load the required data
  data(iris)
  groups <- unique(iris$Species)
  group_by <- "Species"
  formula <- Sepal.Length ~ Sepal.Width + Petal.Length
  fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
  new_formula <- Sepal.Length ~ Sepal.Width 
  results <- updateParallel(new_formula, fitted_models, n.cores = 1)
  expect_is(results, "list")
  expect_equal(length(results), length(fitted_models))
  expect_is(results$setosa, "glmmTMB")

  #uncorrect formula 
  new_formula <- Sepal.Length ~ blabla
  expect_error(updateParallel(new_formula, fitted_models, n.cores = 1))
  
  # Additional parameters: 
   #change family + formula
  new_formula <- Sepal.Length ~ Sepal.Width 
  updated_model <- suppressWarnings(updateParallel(fitted_models, 
                                                    formula = new_formula,
                                                    n.cores = 1,
                                                    family = glmmTMB::nbinom1(link = "log") ))
  expect_s3_class(updated_model$setosa$call$family, "family")
  expect_equal(updated_model$setosa$call$formula, new_formula)
  #change control settings
  updated_model <- suppressWarnings(updateParallel(fitted_models, 
                                                 formula = new_formula, 
                                                 family = glmmTMB::nbinom1(link = "log"), 
                                                  n.cores = 1,
                                                control = glmmTMB::glmmTMBControl(optCtrl=list(iter.max=1e3,
                                                                                               eval.max=1e3))))
  expect_equal(updated_model$setosa$call$control,  glmmTMB::glmmTMBControl(optCtrl=list(iter.max=1e3,eval.max=1e3)))
  
  # Update an updated model
  updated_updated_model <- suppressWarnings(updateParallel(updated_model, 
                                                 formula = new_formula, 
                                                  n.cores = 1,
                                                 family = glmmTMB::ziGamma(link = "inverse")))
  expect_s3_class(updated_updated_model$setosa$call$family,  "family")
  
  
  ## -- update label reference
  data("mtcars")
  # -- Specify categorical variables
  mtcars$vs <- factor(mtcars$vs) ## Engine (0 = V-shaped, 1 = straight)
  levels(mtcars$vs) <- c("V-shaped", "straight")
  mtcars$am <- factor(mtcars$am) ## Transmission (0 = automatic, 1 = manual)
  levels(mtcars$am) <- c("automatic", "manual")
  # -- For each group of number of cylinders:
  # -- Explain fuel consumption with engine shape, Gross horsepower, and transmission type 
  list_tmb <- fitModelParallel(formula = mpg ~ hp + vs + am, data = mtcars, group_by = "cyl", n.cores = 1)
  # -- Relevel transmission and engine shape variables
  reference_labels <- c("straight", "manual")
  result <- updateParallel(formula = mpg ~ hp + vs + am, list_tmb = list_tmb, reference_labels = reference_labels, n.cores = 1)
  
  # Check if the returned list has the same length as the mock list
  expect_equal(length(result), length(list_tmb))
  expect_equal(levels(result[["6"]]$frame$am)[1] , "manual")
  expect_equal(levels(result[["4"]]$frame$am)[1] , "manual")
  expect_equal(levels(result[["6"]]$frame$vs)[1] , "straight")
  expect_equal(levels(result[["4"]]$frame$vs)[1] , "straight")

})



# Test for relevel_list_tmb_frame function
test_that("relevel_list_tmb_frame re-levels categorical variables correctly", {
  data("mtcars")
  # -- Specify categorical variables
  mtcars$vs <- factor(mtcars$vs) ## Engine (0 = V-shaped, 1 = straight)
  levels(mtcars$vs) <- c("V-shaped", "straight")
  mtcars$am <- factor(mtcars$am) ## Transmission (0 = automatic, 1 = manual)
  levels(mtcars$am) <- c("automatic", "manual")
  # -- For each group of number of cylinders:
  # -- Explain fuel consumption with engine shape, Gross horsepower, and transmission type 
  list_tmb <- fitModelParallel(formula = mpg ~ hp + vs + am, data = mtcars, group_by = "cyl", n.cores = 1)
  # -- Relevel transmission and engine shape variables
  reference_labels <- c("straight", "manual")
  result <- relevel_list_tmb_frame(list_tmb, c("vs", "am"), reference_labels)
  
  # Check if the returned list has the same length as the mock list
  expect_equal(length(result), length(list_tmb))
  # Check if categorical variables have been re-leveled correctly
  expect_equal(levels(result[["6"]]$frame$am)[1] , "manual")
  expect_equal(levels(result[["4"]]$frame$am)[1] , "manual")
  expect_equal(levels(result[["6"]]$frame$vs)[1] , "straight")
  expect_equal(levels(result[["4"]]$frame$vs)[1] , "straight")
})

# Test for relevelling_factors function
test_that("relevelling_factors relevel categorical variables correctly", {
  data("mtcars")
  # -- Specify categorical variables
  mtcars$vs <- factor(mtcars$vs) ## Engine (0 = V-shaped, 1 = straight)
  levels(mtcars$vs) <- c("V-shaped", "straight")
  mtcars$am <- factor(mtcars$am) ## Transmission (0 = automatic, 1 = manual)
  levels(mtcars$am) <- c("automatic", "manual")
  # -- For each group of number of cylinders:
  # -- Explain fuel consumption with engine shape, Gross horsepower, and transmission type 
  list_tmb <- fitModelParallel(formula = mpg ~ hp + vs + am, data = mtcars, group_by = "cyl", n.cores = 1)
  # -- Relevel transmission and engine shape variables
  reference_labels <- c("straight", "manual")
  result <- relevelling_factors(list_tmb, reference_labels)
  
   # Check if the returned list has the same length as the mock list
  expect_equal(length(result), length(list_tmb))
  # Check if categorical variables have been re-leveled correctly
  expect_equal(levels(result[["6"]]$frame$am)[1] , "manual")
  expect_equal(levels(result[["4"]]$frame$am)[1] , "manual")
  expect_equal(levels(result[["6"]]$frame$vs)[1] , "straight")
  expect_equal(levels(result[["4"]]$frame$vs)[1] , "straight")
})



# Test for detect_categoricals_vars function
test_that("detect_categoricals_vars detect categorical variables correctly", {
  data("mtcars")
  # -- Specify categorical variables
  mtcars$vs <- factor(mtcars$vs) ## Engine (0 = V-shaped, 1 = straight)
  levels(mtcars$vs) <- c("V-shaped", "straight")
  mtcars$am <- factor(mtcars$am) ## Transmission (0 = automatic, 1 = manual)
  levels(mtcars$am) <- c("automatic", "manual")
  # -- For each group of number of cylinders:
  # -- Explain fuel consumption with engine shape, Gross horsepower, and transmission type 
  list_tmb <- fitModelParallel(formula = mpg ~ hp + vs + am, data = mtcars, group_by = "cyl", n.cores = 1)
  # -- Relevel transmission and engine shape variables
  reference_labels <- c("straight", "manual")
  result <- detect_categoricals_vars(list_tmb[["6"]]$frame, reference_labels)
  
  # Check if the returned list has the same length as the mock list
  expect_equal(length(result), length(reference_labels))
  # Check if categorical variables have been re-leveled correctly
  expect_equal(result, c("vs", "am"))
})


# Test parallel_update function
test_that("parallel_update function returns correct results", {
# Load the required data
  data(iris)
  groups <- unique(iris$Species)
  group_by <- "Species"
  formula <- Sepal.Length ~ Sepal.Width + Petal.Length
  fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
  new_formula <- Sepal.Length ~ Sepal.Width 
  results <- parallel_update(new_formula, fitted_models, n.cores = 1)
  expect_is(results, "list")
  expect_equal(length(results), length(fitted_models))
  expect_is(results$setosa, "glmmTMB")

  #uncorrect formula 
  new_formula <- Sepal.Length ~ blabla
  results <- parallel_update(new_formula, fitted_models, n.cores = 1)
  expect_is(results, "list")
  expect_equal(length(results), length(fitted_models))
  expect_equal(results$setosa, NULL)
  
  # Additional parameters: 
   #change family + formula
  new_formula <- Sepal.Length ~ Sepal.Width 
  updated_model <- suppressWarnings(parallel_update(fitted_models, 
                                                     formula = new_formula,
                                                      n.cores = 1,
                                                      family = glmmTMB::nbinom1(link = "log") ))
  expect_s3_class(updated_model$setosa$call$family, "family")
  expect_equal(updated_model$setosa$call$formula, new_formula)
  #change control
  updated_model <- suppressWarnings(parallel_update(fitted_models, 
                                                 formula = new_formula, 
                                                  n.cores = 1,
                                                 family = glmmTMB::nbinom1(link = "log"), 
                                                control = glmmTMB::glmmTMBControl(optCtrl=list(iter.max=1e3,
                                                                                               eval.max=1e3))))
  expect_equal(updated_model$setosa$call$control,  glmmTMB::glmmTMBControl(optCtrl=list(iter.max=1e3,eval.max=1e3)))
})

# Test fitUpdate function
test_that("fitUpdate function returns correct results", {
  #Load the required data
  data(iris)
  groups <- unique(iris$Species)
  group_by <- "Species"
  formula <- Sepal.Length ~ Sepal.Width + Petal.Length
  fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
  new_formula <- Sepal.Length ~ Sepal.Width 

  updated_model <- fitUpdate("setosa",fitted_models[[1]], new_formula)
  expect_is(updated_model, "glmmTMB")
  
  ## -- check groupId presence
  expect_equal(updated_model$groupId, "setosa")
  
  # Additional parameters: 
   #change family + formula
  updated_model <- suppressWarnings(fitUpdate("setosa", fitted_models[[1]], new_formula, 
                                              family = glmmTMB::nbinom1(link = "log") ))
  expect_s3_class(updated_model$call$family, "family")
  expect_equal(updated_model$call$formula, new_formula)
  #change control
  updated_model <- suppressWarnings(fitUpdate("setosa", fitted_models[[1]], 
                                              new_formula, 
                                              family = glmmTMB::nbinom1(link = "log"), 
                                              control = glmmTMB::glmmTMBControl(optCtrl=list(iter.max=1e3,
                                                                                               eval.max=1e3))))
  expect_equal(updated_model$call$control,  glmmTMB::glmmTMBControl(optCtrl=list(iter.max=1e3,eval.max=1e3)))
  
})


# Test launchUpdate function
test_that("launchUpdate function returns correct results", {
  data(iris)
  groups <- unique(iris$Species)
  group_by <- "Species"
  formula <- Sepal.Length ~ Sepal.Width + Petal.Length
  fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
  new_formula <- Sepal.Length ~ Sepal.Width 
  updated_model <- launchUpdate(fitted_models[[1]], new_formula)
  expect_is(updated_model, "glmmTMB")
  # Additional parameters: 
   #change family + formula
  updated_model <- launchUpdate(fitted_models[[1]], new_formula, family = glmmTMB::nbinom1(link = "log") )
  expect_s3_class(updated_model$call$family, "family")
  expect_equal(updated_model$call$formula, new_formula)
  #change control
  updated_model <- launchUpdate(fitted_models[[1]], new_formula, family = glmmTMB::nbinom1(link = "log"), 
                                control = glmmTMB::glmmTMBControl(optimizer=optim, optArgs=list(method="BFGS")))
  expect_equal(updated_model$call$control,  glmmTMB::glmmTMBControl(optimizer=optim, optArgs=list(method="BFGS")))
  
})

