# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand


# Test for first_non_null_index function
test_that("first_non_null_index returns the correct index", {
  lst <- list(NULL, NULL, 3, 5, NULL)
  expect_equal(first_non_null_index(lst), 3)
})



# Test unitaires pour la fonction join_dtf
test_that("join_dtf réalise la jointure correctement", {
  # Création de données de test
  df1 <- data.frame(id = 1:5, value = letters[1:5])
  df2 <- data.frame(id = 1:5, category = LETTERS[1:5])
  
  # Exécution de la fonction
  result <- join_dtf(df1, df2, "id", "id")
  
  # Vérification des résultats
  expect_true(is.data.frame(result))
  expect_equal(nrow(result), 5)
  expect_equal(ncol(result), 3)
  expect_equal(names(result), c("id", "value", "category"))
  expect_true(all.equal(result$id, df1$id))
  expect_true(all.equal(result$id, df2$id))
})


test_that("clean_variable_name correctly removes digits, spaces, and special characters", {
  expect_equal(clean_variable_name("my variable name"), "myvariablename")
  expect_equal(clean_variable_name("variable_1"), "variable")
  expect_equal(clean_variable_name("^spec(ial#chars! "), "specialchars")
})

test_that("clean_variable_name handles reserved names properly", {
  expect_error(clean_variable_name("interactions"))
  expect_error(clean_variable_name("correlations"))
})


test_that("getLabels generates labels for variables", {
  labels <- getLabels(c("varA", "varB"), c(2, 3))
  expect_equal(length(labels), 2)
  expect_equal(length(labels[[1]]), 2)
  expect_equal(length(labels[[2]]), 3)
})

test_that("getGridCombination generates a grid of combinations", {
  labels <- list(A = c("A1", "A2"), B = c("B1", "B2", "B3"))
  grid_combination <- getGridCombination(labels)
  expect_equal(dim(grid_combination), c(6, 2))
})


test_that("generateGridCombination_fromListVar returns expected output", {
  result <- generateGridCombination_fromListVar(init_variable())
  expect <- data.frame(label_myVariable = c("myVariable1", "myVariable2"))
  expect_equal(nrow(result), nrow(expect))
  expect_equal(ncol(result), ncol(expect))
})

# Tests for convert2Factor
test_that("convert2Factor converts specified columns to factors", {
  # Create a sample data frame
  data <- data.frame(
    Category1 = c("A", "B", "A", "B"),
    Category2 = c("X", "Y", "X", "Z"),
    Value = 1:4,
    stringsAsFactors = FALSE
  )
  
  # Convert columns to factors
  result <- convert2Factor(data, columns = c("Category1", "Category2"))
  
  # Check the output
  expect_is(result$Category1, "factor")  # Category1 column converted to factor
  expect_is(result$Category2, "factor")  # Category2 column converted to factor
})

test_that("removeDuplicatedWord returns expected output", {
  words <- c("hellohello", "worldworld", "programmingprogramming", "R isis great")
  cleaned_words <- removeDuplicatedWord(words)
  expect_equal(cleaned_words, c("hello", "world", "programming", "R is great"))
})

test_that("reorderColumns returns expected output",{
    df <- data.frame(A = 1:3, B = 4:6, C = 7:9)
    # Define the desired column order
    columnOrder <- c("B", "C", "A")
    # Reorder the columns of the dataframe
    df_reorder <- reorderColumns(df, columnOrder)
    expect_equal(colnames(df_reorder), columnOrder)
    expect_equal(dim(df_reorder), dim(df))

})


test_that( "generateGridCombination_fromListVar return expected output", {
    ## case 1
    gridcom <- generateGridCombination_fromListVar(init_variable())
    expect_s3_class(gridcom, "data.frame")
    expect_equal(gridcom$label_myVariable, factor(c("myVariable1", "myVariable2")))

    ## case 2
    init_variables <- init_variable() %>% init_variable(name = "var" , mu = 2, sd = 1, level = 3) 
    gridcom <- generateGridCombination_fromListVar(init_variables)
    expect_s3_class(gridcom, "data.frame")
    expect_equal(unique(gridcom$label_myVariable), factor(c("myVariable1", "myVariable2")))
    expect_equal(unique(gridcom$label_var), factor(c("var1", "var2", "var3")))

  })

test_that( "getGivenAttribute return expected output", {
  ## -- case 1
  level_attr <- getGivenAttribute(init_variable(), "level")
  expect_equal(level_attr$myVariable, 2)

  ## -- case 2
  init_variables <- init_variable() %>% init_variable(name = "var" , mu = 2, sd = 1, level = 3) 
  mu_attr <- getGivenAttribute(init_variables, "mu")
  expect_equal(mu_attr$var, 2)
} )



test_that("isValidList_tmb function", {
  # Test with a valid list of glmmTMB objects
  l_tmb <- list("model1" = glmmTMB::glmmTMB(mpg ~ hp + vs + am + (1|cyl), data = mtcars),
                 "model2" = glmmTMB::glmmTMB(mpg ~ hp + vs + am + (1|cyl), data = mtcars))
  expect_true(isValidList_tmb(l_tmb))
  
  # Test with a list containing NULL elements
  expect_error(isValidList_tmb(list(NULL, NULL)), "All elements in 'list_tmb' are NULL")
  
  # Test with an empty list
  expect_error(isValidList_tmb(list()), "All elements in 'list_tmb' are NULL")
})

test_that("isValidGlmmTmb function", {
  # Test with a valid glmmTMB object
  valid_model <- glmmTMB::glmmTMB(mpg ~ hp + vs + am + (1|cyl), data = mtcars)
  expect_true(isValidGlmmTmb("model", valid_model))
  
  # Test with an invalid object (not a glmmTMB object)
  invalid_object <- list(a = 1, b = 2)
  expect_error(isValidGlmmTmb("object", invalid_object), "Element object is not a glmmTMB object.")
  
  # Test with NULL object
  expect_true(isValidGlmmTmb("model", NULL))
})



# Mock object

test_that("isValidMock_obj checks if the provided object is a valid mock object", {
  mock_obj <- mock_rnaseq(init_variable(), n_genes = 100, 4, 4)

  # Test with a valid mock object
  expect_true(isValidMock_obj(mock_obj))
  
  # Test with an object missing an element
  missing_element_obj <- list(settings = list(), init = list(), groundTruth = list(), counts = list())
  expect_error(isValidMock_obj(missing_element_obj))
  
  # Test with an object containing additional elements
  additional_element_obj <- mock_obj
  additional_element_obj$error_name <- list()
  expect_warning(isValidMock_obj(additional_element_obj))
})

